@@ -7,6 +7,7 @@ from sentence_transformers import SentenceTransformer
 
 from semantic_code_search.embed import do_embed
 from semantic_code_search.query import do_query
+from semantic_code_search.cluster import do_cluster
 
 
 def git_root(path=None):
@@ -37,6 +38,11 @@ def query_func(args):
     do_query(args, model)
 
 
+def cluster_func(args):
+    model = SentenceTransformer(args.model_name_or_path)
+    do_cluster(args, model)
+
+
 def main():
     parser = argparse.ArgumentParser(
         prog='sem', description='Search your codebase using natural language')
@@ -55,6 +61,16 @@ def main():
                         required=False, default=5, help='Number of results to return')
     parser.add_argument('-e', '--editor', choices=[
                         'vscode', 'vim'], default='vscode', required=False, help='Editor to open selected result in')
+    parser.add_argument('-c', '--cluster', action='store_true', default=False,
+                        required=False, help='Generate clusters of related functions and methods')
+    parser.add_argument('--cluster-max-distance', metavar='THRESHOLD', type=float, default=0.2, required=False,
+                        help='How close functions need to be to one another to be clustered. Distance 0 means that the code is identical, smaller values (e.g. 0.2, 0.3) are stricter and result in fewer matches ')
+    parser.add_argument('--cluster-min-lines', metavar='SIZE', type=int, default=0, required=False,
+                        help='Ignore clusters with code snippets smaller than this size (lines of code). Use this if you are not interested in smaller duplications (eg. one liners)')
+    parser.add_argument('--cluster-min-cluster-size', metavar='SIZE', type=int, default=2, required=False,
+                        help='Ignore clusters smaller than this size. Use this if you want to find code that is similar and repeated many times (e.g. >5)')
+    parser.add_argument('--cluster-ignore-identincal', action='store_true', default=True,
+                        required=False, help='Ignore identical code / exact duplicates (where distance is 0)')
     parser.set_defaults(func=query_func)
     parser.add_argument('query_text', nargs=argparse.REMAINDER)
 
@@ -62,6 +78,8 @@ def main():
 
     if args.embed:
         embed_func(args)
+    elif args.cluster:
+        cluster_func(args)
     else:
         query_func(args)
 
